@@ -3,14 +3,16 @@
 module Network.DNS.Encode (
     encode
   , composeQuery
+  , composeQueryAD
   ) where
 
-import qualified Blaze.ByteString.Builder as BB
 import Control.Monad (when)
 import Control.Monad.State (State, modify, execState, gets)
 import Data.Binary (Word16)
 import Data.Bits ((.|.), bit, shiftL, setBit)
+import qualified Data.ByteString.Builder as BB
 import qualified Data.ByteString.Char8 as BS
+import qualified Data.ByteString.Lazy as BL
 import Data.ByteString.Lazy.Char8 (ByteString)
 import Data.IP (IP(..),fromIPv4, fromIPv6b)
 import Data.List (dropWhileEnd)
@@ -37,6 +39,21 @@ composeQuery idt qs = encode qry
       , question = qs
       }
 
+composeQueryAD :: Int -> [Question] -> ByteString
+composeQueryAD idt qs = encode qry
+  where
+      hdr = header defaultQuery
+      flg = flags hdr
+      qry = defaultQuery {
+          header = hdr {
+              identifier = idt,
+              flags = flg {
+                  authenData = True
+              }
+           }
+        , question = qs
+        }
+
 ----------------------------------------------------------------
 
 -- | Composing DNS data.
@@ -83,6 +100,7 @@ encodeFlags DNSFlags{..} = put16 word
     st :: State Word16 ()
     st = sequence_
               [ set (word16 rcode)
+              , when authenData          $ set (bit 5)
               , when recAvailable        $ set (bit 7)
               , when recDesired          $ set (bit 8)
               , when trunCation          $ set (bit 9)
@@ -101,10 +119,11 @@ encodeQuestion Question{..} = encodeDomain qname
 putRData :: RData -> SPut
 putRData rd = do
     addPositionW 2 -- "simulate" putInt16
-    rDataWrite <- encodeRDATA rd
-    let rdataLength = fromIntegral . BS.length . BB.toByteString . BB.fromWrite $ rDataWrite
-    let rlenWrite = BB.writeInt16be rdataLength
-    return rlenWrite <> return rDataWrite
+    rDataBuilder <- encodeRDATA rd
+    -- fixmed: SPut must hold length
+    let rdataLength = fromIntegral . BL.length . BB.toLazyByteString $ rDataBuilder
+    let rlenBuilder = BB.int16BE rdataLength
+    return rlenBuilder <> return rDataBuilder
 
 encodeRR :: ResourceRecord -> SPut
 encodeRR ResourceRecord{..} = mconcat [ encodeDomain rrname
@@ -150,6 +169,12 @@ encodeRDATA rd = case rd of
         , putInt16 port
         , encodeDomain dom
         ]
+    (RD_TLSA u s m d) -> mconcat
+        [ putInt8 u
+        , putInt8 s
+        , putInt8 m
+        , putByteString d
+        ]
 
 encodeOData :: OData -> SPut
 encodeOData (OD_ClientSubnet srcNet scpNet ip) = let dropZeroes = dropWhileEnd (==0)
